import numpy as np


def fish():
    """Returns a scatterer phantom for ultrasound simulation tests.

    Returns:
        ndarray: The scatterer positions of shape (n_scat, 3).
    """
    # The size is the height of the fish
    size = 11e-3
    z_offset = 2.0 * size

    # See https://en.wikipedia.org/wiki/Fish_curve
    def fish_curve(t, size=1):
        x = size * (np.cos(t) - np.sin(t) ** 2 / np.sqrt(2))
        y = size * np.cos(t) * np.sin(t)
        return x, y

    scat_x, scat_z = fish_curve(np.linspace(0, 2 * np.pi, 100), size=size)

    scat_x = np.concatenate(
        [
            scat_x,
            np.array([size * 0.7]),
            np.array([size * 1.1]),
            np.array([size * 1.4]),
            np.array([size * 1.2]),
        ]
    )
    scat_y = np.zeros_like(scat_x)
    scat_z = np.concatenate(
        [
            scat_z,
            np.array([-size * 0.1]),
            np.array([-size * 0.25]),
            np.array([-size * 0.6]),
            np.array([-size * 1.0]),
        ]
    )

    scat_z += z_offset
    return np.stack([scat_x, scat_y, scat_z], axis=1)



import matplotlib.pyplot as plt
import numpy as np
from matplotlib import animation

from zea import Scan


def animate_images(
    images, path, scan: Scan = None, interval=100, cmap="gray", figsize=(5, 4), dpi=80
):
    """Helper function to animate a list of images."""
    if interval <= 0:
        raise ValueError("interval must be a positive integer (milliseconds).")
    if len(images) == 0:
        raise ValueError("images must be a non-empty sequence.")
    fig, ax = plt.subplots(figsize=figsize, dpi=dpi)
    if scan is not None:
        extent = scan.extent * 1e3 if getattr(scan, "extent") is not None else None
    else:
        extent = None
    im = ax.imshow(np.array(images[0]), animated=True, cmap=cmap, extent=extent)
    ax.set_xlabel("X (mm)")
    ax.set_ylabel("Z (mm)")

    def update(frame):
        im.set_array(np.array(images[frame]))
        return [im]

    ani = animation.FuncAnimation(
        fig,
        update,
        frames=len(images),
        blit=True,
        interval=interval,
    )
    plt.close(fig)
    fps = max(1, 1000 // interval)

    ani.save(path, writer="imagemagick", fps=fps)
